<?php

namespace Drupal\password_notification\Form;

use Drupal\Component\Utility\EmailValidatorInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Password Changed history page.
 */
class PasswordChangedHistoryForm extends FormBase {
  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;
  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;
  /**
   * The email validator.
   *
   * @var \Drupal\Component\Utility\EmailValidatorInterface
   */
  protected $emailValidator;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'password_changed_history_form';
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(Connection $database, RequestStack $request_stack, EmailValidatorInterface $email_validator) {

    $this->database = $database;
    $this->requestStack = $request_stack;
    $this->emailValidator = $email_validator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('request_stack'),
      $container->get('email.validator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, UserInterface $user = NULL) {
    $request = $this->requestStack->getCurrentRequest();
    $form = [];
    $username = $email = $start_date = $end_date = '';
    // Configure the table header columns.
    $header = [
    ['data' => 'Name', 'field' => 'username'],
    ['data' => 'Email', 'field' => 'email'],
    ['data' => 'User Id', 'field' => 'user_id'],
    ['data' => 'Changed by(User Id)', 'field' => 'changed_by_user_id'],
    ['data' => 'Changed by(Name)', 'field' => 'changed_by_user_name'],
    ['data' => 'IP Address', 'field' => 'ip_address'],
    ['data' => 'Mail Status', 'field' => 'mail_send'],
    ['data' => 'Date', 'field' => 'created', 'sort' => 'desc'],
    ];
    // Select Query from Password Changed History Table.
    $query = $this->database->select('password_changed_history', 'p')
      ->extend('Drupal\Core\Database\Query\TableSortExtender')
      ->extend('Drupal\Core\Database\Query\PagerSelectExtender');
    if (is_object($user)) {
      $query->condition('p.user_id', $user->get('uid')->value, '=');
    }
    // Filter by Username.
    if (!empty($request->query->get('username'))) {
      $username = $request->query->get('username');
      $query->condition('p.username', '%' . $this->database->escapeLike($username) . '%', 'LIKE');
    }
    // Filter by Email.
    if (!empty($request->query->get('email'))) {
      $email = $request->query->get('email');
      $query->condition('p.email', $email, '=');
    }
    if (!empty($request->query->get('start_date')) && !empty($request->query->get('end_date'))) {
      $start_date = $request->query->get('start_date');
      $start_date_timestamp = strtotime($start_date . '00:00:00');
      $end_date = $request->query->get('end_date');
      $end_date_timestamp = strtotime($end_date . '23:59:59');
      $query->condition('p.created', [$start_date_timestamp, $end_date_timestamp], 'BETWEEN');
    }
    $query->fields('p', ['username',
      'email',
      'user_id',
      'changed_by_user_id',
      'changed_by_user_name',
      'ip_address',
      'mail_send',
      'created',
    ])
      ->limit(10)
      ->orderByHeader($header);

    $results = $query->execute();
    if (!empty($results)) {
      $rows = [];
      foreach ($results as $row) {
        $mail_status = ($row->mail_send == 'TRUE') ? $this->t('Email delivery successful') : $this->t('Email delivery unsuccessful');
        $rows[] = [$row->username,
          $row->email,
          $row->user_id,
          $row->changed_by_user_id,
          $row->changed_by_user_name,
          $row->ip_address,
          $mail_status,
          date('d-m-Y h:i A', $row->created),
        ];
      }
      /* Filters Code Start */
      $form['#attached']['library'][] = 'password_notification/default';
      $form['filter'] = [
        '#type' => 'details',
        '#title' => $this->t('Filter option'),
        '#collapsible' => TRUE,
        '#prefix' => '<div class="filter-parent-wrap">',
        '#collapsed' => TRUE,
        '#open' => TRUE,
      ];
      if (!$user) {
        $form['filter']['username'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Username'),
          '#default_value' => $username,
          '#prefix' => '<div class="filter-fields-wrap">',
          '#size' => 20,
        ];
        $form['filter']['email'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Email'),
          '#default_value' => $email,
          '#size' => 20,
        ];
      }
      $form['filter']['start_date'] = [
        '#type' => 'date',
        '#title' => $this->t('Start Date'),
        '#default_value' => $start_date,
        '#size' => 20,
      ];
      if (is_object($user)) {
        $form['filter']['start_date']['#prefix'] = '<div class="filter-fields-wrap">';
      }
      $form['filter']['end_date'] = [
        '#type' => 'date',
        '#title' => $this->t('End Date'),
        '#default_value' => $end_date,
        '#suffix' => '</div>',
        '#size' => 20,
      ];
      $form['filter']['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t('Apply'),
        '#prefix' => '<div class="filter-btn-wrap">',
      ];
      $form['filter']['reset'] = [
        '#type' => 'submit',
        '#value' => $this->t('Reset'),
        '#submit' =>
      [[$this, 'reseFormSubmit']],
        '#limit_validation_errors' => [],
        '#suffix' => '</div></div>',
      ];
      /* Filters Code end */
      $form['table'] = [
        '#theme' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#empty' => $this->t('No Password Changed history available.'),
      ];
      $form['pager'] = ['#type' => 'pager'];
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Email Validation.
    if (!empty($form_state->getValue('email'))) {
      if (!$this->emailValidator->isValid($form_state->getValue('email'))) {
        $form_state->setErrorByName('email', $this->t('Please Provide Valid Email Address.'));
      }
    }
    /* Date Filter Validation Code Start */
    // Start Date Validation.
    if (!empty($form_state->getValue('end_date'))) {
      if (!$form_state->getValue('start_date')) {
        $form_state->setErrorByName('start_date', $this->t('Please Provide Start Date.'));
      }
    }
    // End Date Validation.
    if (!empty($form_state->getValue('start_date'))) {
      if (!$form_state->getValue('end_date')) {
        $form_state->setErrorByName('end_date', $this->t('Please Provide End Date.'));
      }
    }
    /* Date Filter Validation Code End */
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $username = $email = $start_date = $end_date = '';
    // Username.
    if (!empty($form_state->getValue('username'))) {
      $username = $form_state->getValue('username');
    }
    // Email.
    if (!empty($form_state->getValue('email'))) {
      $email = $form_state->getValue('email');
    }
    // Start date.
    if (!empty($form_state->getValue('start_date'))) {
      $start_date = $form_state->getValue('start_date');
    }
    // End date.
    if (!empty($form_state->getValue('end_date'))) {
      $end_date = $form_state->getValue('end_date');
    }
    $get_route_name = $this->requestStack->getCurrentRequest()->get('_route');
    ;
    $route_param = [];
    $query_param = [];
    switch ($get_route_name) {
      case 'password_notification.report':
        if (!empty($username) || !empty($email) || !empty($start_date) || !empty($end_date)) {
          $query_param = [
            'query' => [
              'username' => $username,
              'email' => $email,
              'start_date' => $start_date,
              'end_date' => $end_date,
            ],
          ];
        }
        break;

      case 'password_notification.user_report':
        $get_current_path = $this->requestStack->getCurrentRequest()->getpathInfo();
        $args = explode('/', $get_current_path);
        $route_param = ['user' => $args[2]];
        if (!empty($start_date) && !empty($end_date)) {
          $query_param = [
            'query' => [
              'start_date' => $start_date,
              'end_date' => $end_date,
            ],
          ];
        }
        break;
    }
    $form_state->setRedirect($get_route_name,
    $route_param,
    $query_param
    );
  }

  /**
   * {@inheritdoc}
   */
  public function reseFormSubmit(array &$form, FormStateInterface $form_state) {
    $get_route_name = $this->requestStack->getCurrentRequest()->get('_route');
    $route_param = [];
    if ($get_route_name == 'password_notification.user_report') {
      $get_current_path = $this->requestStack->getCurrentRequest()->getpathInfo();
      $args = explode('/', $get_current_path);
      $route_param = ['user' => $args[2]];
    }
    $form_state->setRedirect($get_route_name, $route_param);
  }

}
